/**
 * Copyright (c) 2008 by The Shoe Shine Boy.
 * 
 * This file is part of Mail Blaster.
 * 
 * Mail Blaster is free software: you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or (at your 
 * option) any later version.
 *
 * Mail Blaster is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with Mail Blaster. If not, see <http://www.gnu.org/licenses/>.
 */
package com.gt.mailblaster.config;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;

/**
 * The class that extends AbstractConfig class and provides SMTP configuration save/load.
 * 
 * @author The Shoe Shine Boy
 *
 */
public class SMTPConfig extends AbstractConfig {
	// SMTP server and email parameters
	private String m_mailHost, m_acName, m_acPwd, m_subject, m_body;
	private boolean m_authenticate, m_enableTLS, m_sendAsHTML;
	private ArrayList m_to = new ArrayList();
	private ArrayList m_attachments = new ArrayList();
	
	/**
	 * This method loads the SMTPConfig class from a DOM Node Object. This method calls itself 
	 * recursively until it has read the whole DOM tree and loaded all the configuration 
	 * parameters
	 * 
	 * @param n				The DOM Node
	 * @throws DOMException
	 * @throws IOException
	 */
	private void load(Node n) throws DOMException, IOException {
		if (n.getNodeName().equals("smtp") || n.getNodeName().equals("tofields") || n.getNodeName().equals("attachments")) {
			NodeList children = n.getChildNodes();
			for (int i=0; i<children.getLength(); i++) {
				load(children.item(i));
			}
		} else if (n.getNodeName().equals("mailhost")) {
			m_mailHost = n.getFirstChild().getNodeValue();
		} else if (n.getNodeName().equals("acname")) {
			m_acName = n.getFirstChild().getNodeValue();
		} else if (n.getNodeName().equals("acpwd")) {
			m_acPwd = new String(new BASE64Decoder().decodeBuffer(n.getFirstChild().getNodeValue()));
		} else if (n.getNodeName().equals("subject")) {
			m_subject = n.getFirstChild().getNodeValue();
		} else if (n.getNodeName().equals("body")) {
			m_body = n.getFirstChild().getNodeValue();
		} else if (n.getNodeName().equals("authenticate")) {
			m_authenticate = Boolean.valueOf(n.getFirstChild().getNodeValue()).booleanValue();
		} else if (n.getNodeName().equals("enabletls")) {
			m_enableTLS = Boolean.valueOf(n.getFirstChild().getNodeValue()).booleanValue();
		} else if (n.getNodeName().equals("sendashtml")) {
			m_sendAsHTML = Boolean.valueOf(n.getFirstChild().getNodeValue()).booleanValue();
		} else if (n.getNodeName().equals("to")) {
			m_to.add(n.getFirstChild().getNodeValue());
		} else if (n.getNodeName().equals("attachment")) {
			m_attachments.add(new File(n.getFirstChild().getNodeValue()));
		} else {
			// skip it
		}
	}
	
	public void loadFrom(File file) throws ConfigException {
		Document doc = null;
		try {
			doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(file);
		} catch (SAXException e1) {
			throw new ConfigException(e1);
		} catch (IOException e1) {
			throw new ConfigException(e1);
		} catch (ParserConfigurationException e1) {
			throw new ConfigException(e1);
		}
		
		try {
			load(doc.getFirstChild());
		} catch (DOMException e) {
			throw new ConfigException(e);
		} catch (IOException e) {
			throw new ConfigException(e);
		}
	}

	public void saveTo(File file) throws ConfigException {
		String str =
			"<smtp>" +
				"<mailhost>" + m_mailHost + "</mailhost>" +
				"<acname>" + m_acName + "</acname>" +
				"<acpwd><![CDATA[" + new BASE64Encoder().encode(m_acPwd.getBytes()) + "]]></acpwd>" +
				"<authenticate>" + m_authenticate + "</authenticate>" +
				"<enabletls>" + m_enableTLS + "</enabletls>" +
				"<sendashtml>" + m_sendAsHTML + "</sendashtml>" +
				"<subject><![CDATA[" + m_subject + "]]></subject>" +
				"<body><![CDATA[" + m_body + "]]></body>" +
				"<tofields>";
		for (int i=0; i<m_to.size(); i++) {
			str = str + "<to>" + m_to.get(i) + "</to>"; 
		}
		str = str +
				"</tofields>" +
				"<attachments>";
		for (int i=0; i<m_attachments.size(); i++) {
			try {
				str = str + "<attachment>" + ((File)m_attachments.get(i)).getCanonicalPath() + "</attachment>";
			} catch (IOException e) {
				throw new ConfigException(e);
			}
		}
		str = str +
				"</attachments>" +
			"</smtp>";
		
		try {
			new FileOutputStream(file).write(str.getBytes());
		} catch (FileNotFoundException e) {
			throw new ConfigException(e);
		} catch (IOException e) {
			throw new ConfigException(e);
		}
	}
	
	//
	// Getter and Setter methods
	//
	public String getMailHost() {
		return m_mailHost;
	}
	public void setMailHost(String str) {
		m_mailHost = str;
	}
	public String getAcName() {
		return m_acName;
	}
	public void setAcName(String str) {
		m_acName = str;
	}
	public String getAcPwd() {
		return m_acPwd;
	}
	public void setAcPwd(String str) {
		m_acPwd = str;
	}
	public String getSubject() {
		return m_subject;
	}
	public void setSubject(String str) {
		m_subject = str;
	}
	public String getBody() {
		return m_body;
	}
	public void setBody(String str) {
		m_body = str;
	}
	public boolean isAuthenticate() {
		return m_authenticate;
	}
	public void setIsAuthenticate(boolean flag) {
		m_authenticate = flag;
	}
	public boolean isEnableTLS() {
		return m_enableTLS;
	}
	public void setIsEnableTLS(boolean flag) {
		m_enableTLS = flag;
	}
	public boolean isSendAsHTML() {
		return m_sendAsHTML;
	}
	public void setIsSendAsHTML(boolean flag) {
		m_sendAsHTML = flag;
	}
	public String[] getTos() {
		String[] tos = new String[m_to.size()];
		for (int i=0; i<tos.length; i++) {
			tos[i] = (String)m_to.get(i);
		}
		return tos;
	}
	public void setTos(String[] tos) {
		m_to.clear();
		if (tos != null) {
			for (int i=0; i<tos.length; i++) {
				m_to.add(tos[i]);
			}
		} else {
			// do nothing
		}
	}
	public File[] getAttachments() {
		File[] files = new File[m_attachments.size()];
		for (int i=0; i<files.length; i++) {
			files[i] = (File)m_attachments.get(i);
		}
		return files;
	}
	public void setAttachments(File[] files) {
		m_attachments.clear();
		if (files != null) {
			for (int i=0; i<files.length; i++) {
				m_attachments.add(files[i]);
			}
		} else {
			// do nothing
		}
	}
}
